package com.qihoo360.mobilesafe.reward.container

import android.os.Handler
import android.os.Message


import com.qihoo360.mobilesafe.support.log.Logger

import java.lang.ref.WeakReference
import java.util.ArrayList

class ContainerWatcher private constructor() {

    private val mHandler = LoopHandler(this)

    private val LISTENERS = ArrayList<WeakReference<ContainerBase>>()

    init {
        mHandler.sendEmptyMessageDelayed(MSG_LOOP,TIME_DELAY.toLong())
    }

    fun addWatch(container: ContainerBase) {
        synchronized(LISTENERS) {
            LISTENERS.add(WeakReference(container))
            Logger.d(TAG, "cached container size:" + LISTENERS.size)
        }
    }

    private fun handleLoop() {
        synchronized(LISTENERS) {
            for (i in LISTENERS.indices.reversed()) {
                val w = LISTENERS[i]
                val container = w.get()
                container?.handleLoop() ?: LISTENERS.removeAt(i)
            }
        }
    }

    private class LoopHandler(outer: ContainerWatcher) : Handler() {

        private val outer: WeakReference<ContainerWatcher> = WeakReference(outer)

        override fun handleMessage(msg: Message) {
            val obj = this.outer.get() ?: return

            when (msg.what) {
                MSG_LOOP -> {
                    removeMessages(MSG_LOOP)
                    obj.handleLoop()
                    sendEmptyMessageDelayed(MSG_LOOP, TIME_DELAY.toLong())
                }
            }
        }

    }

    companion object {

        private val TAG = "ContainerWatcher"

        private var sInstance: ContainerWatcher? = null

        const val MSG_LOOP = 0

        const val TIME_DELAY = 500

        val instance: ContainerWatcher
            get() {
                if (sInstance == null) {
                    sInstance = ContainerWatcher()
                }
                return sInstance as ContainerWatcher
            }
    }
}
